package com.taauav.common.util;

import lombok.extern.slf4j.Slf4j;
import org.springframework.util.StringUtils;

import java.io.*;
import java.util.List;
import java.util.Map;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

/**
 * <p>文件复制、删除、剪切封装类</p>
 *
 * @author : dyg
 * @date : 2019-11-25 10:09
 **/
@Slf4j
public class FileUtil {

    public static boolean copyGeneralFile(String srcPath, String destDir) {
        boolean flag = false;
        File file = new File(srcPath);
        if (!file.exists()) {
            log.warn(srcPath+"不存在");
            return flag;
        }
        if (file.isFile()) {
            flag = copyFile(srcPath, destDir);
        } else if (file.isDirectory()) {
            flag = copyDirectory(srcPath, destDir);
        }
        return flag;
    }

    /**
     * 复制文件
     * @param srcPath
     * @param destDir
     * @return
     */
    private static boolean copyFile(String srcPath, String destDir) {
        boolean flag = false;
        File srcFile = new File(srcPath);
        if (!srcFile.exists()) {
            return false;
        }
        File destDirFile = new File(destDir);
        if (!destDirFile.isDirectory()) {
            destDirFile.mkdirs();
        }
        String fileName = srcFile.getName();
        if (fileName.contains("thumbnail")) {
            fileName = fileName.replace("-thumbnail", "");
        }
        String destPath = destDir + fileName;
        if (destPath.equals(srcPath)) {
            log.info("源文件路径和目标文件路径重复");
            return false;
        }
        File destFile = new File(destPath);
        if (destFile.exists() && destFile.isFile()) {
            log.info("目标目录下已有同名文件!");
            return false;
        }
        FileInputStream fis;
        FileOutputStream fos;
        try {
            fis = new FileInputStream(srcFile);
            fos = new FileOutputStream(destFile);
            byte[] buff = new byte[1024];
            int c;
            while ((c = fis.read(buff)) != -1) {
                fos.write(buff,0,c);
            }
            fis.close();
            fos.close();
            flag = true;
        } catch (IOException e) {
            log.error(e.getMessage());
        }
        return flag;
    }

    /**
     * 复制文件夹
     * @param srcPath
     * @param destDir
     * @return
     */
    private static boolean copyDirectory(String srcPath, String destDir) {
        boolean flag = false;
        File srcFile = new File(srcPath);
        if (!srcFile.exists()) {
            log.info("源文件夹不存在");
            return false;
        }
        //获得待复制的文件夹的名字
        String dirName = getDirName(srcPath);
        //目标文件夹的完整路径
        String destPath = destDir + File.separator + dirName;
        if (destPath.equals(srcPath)) {
            log.info("目标文件夹与源文件夹重复");
        }
        File destDirFile = new File(destPath);
        destDirFile.mkdirs();
        File[] fileList = srcFile.listFiles();
        if (fileList.length == 0) {
            flag = true;
        } else {
            for (File temp : fileList) {
                if (temp.isFile()) {
                    flag = copyFile(temp.getAbsolutePath(),destPath);
                } else if (temp.isDirectory()) {
                    flag = copyGeneralFile(temp.getAbsolutePath(),destPath);
                }
                if (!flag) {
                    break;
                }
            }
        }
        return true;
    }

    public static boolean deleteGeneralFile(String path) {
        boolean flag = false;
        File file = new File(path);
        if (!file.exists()) {
            log.info("删除的文件不存在");
            return false;
        }
        if (file.isDirectory()) {
            flag = deleteDirectory(file.getAbsolutePath());
        } else {
            flag = deleteFile(file);
        }
        return flag;
    }

    /**
     * 删除文件
     * @param file
     * @return
     */
    private static boolean deleteFile(File file) {
        return file.delete();
    }

    /**
     * 删除文件夹
     * @param path
     * @return
     */
    private static boolean deleteDirectory(String path) {
        boolean flag = true;
        File dirFile = new File(path);
        if (!dirFile.isDirectory()) {
            return false;
        }
        File[] files = dirFile.listFiles();
        for (File file : files) {
            if (file.isFile()) {
                flag = deleteFile(file);
            } else if(file.isDirectory()) {
                flag = deleteDirectory(file.getAbsolutePath());
            }
            if (!flag) {
                break;
            }
        }
        flag = dirFile.delete();//删除空目录
        return flag;
    }

    /**
     * 剪切方法：复制+删除
     * @param srcPath
     * @param destDir
     * @return
     */
    public static boolean cutGeneralFile(String srcPath, String destDir) {
        if (!copyGeneralFile(srcPath,destDir)) {
            return false;
        }
        if (!deleteGeneralFile(srcPath)) {
            return false;
        }
        return true;
    }
    /**
     * 获取待复制文件夹的文件名
     * @param dir
     * @return
     */
    private static String getDirName(String dir) {
        if (dir.endsWith(File.separator)) {
            dir = dir.substring(0, dir.lastIndexOf(File.separator));
        }
        return dir.substring(dir.lastIndexOf(File.separator) + 1);
    }

    /**
     * 通过指定路径和文件名来获取文件对象，当文件不存在时自动创建
     * @param path
     * @param fileName
     * @return
     * @throws IOException
     */
    public static File getFile(String path, String fileName) throws IOException {
        File file;
        if (path != null && !path.equals("")) {
            file = new File(path,fileName);
        } else {
            file = new File(fileName);
        }
        File parentFile = new File(file.getParent());
        if (!parentFile.isDirectory()) {
            parentFile.mkdirs();
        }
        if (!file.exists()) {
            file.createNewFile();
        }
        return file;
    }

    /**
     * 获得指定文件的输出流
     * @param file
     * @return
     * @throws FileNotFoundException
     */
    public static FileOutputStream getFileStream(File file) throws FileNotFoundException {
        return new FileOutputStream(file);
    }

    /**
     * 将多个文件压缩
     * @param fileList<Map<String,Object>> file:File,foleder:"folder"
     * @param zipFileName
     * @return
     * @throws IOException
     */
    public static File getZipFile(List<Map<String,Object>> fileList, String zipFileName) throws IOException {
        File zipFile = getFile("", zipFileName);
        FileOutputStream outputStream = getFileStream(zipFile);
        ZipOutputStream zipOutputStream = new ZipOutputStream(outputStream);

        int size = fileList.size();
        for (int i=0; i<size; i++) {
            Map<String,Object> map = fileList.get(i);
            File file = (File)map.get("file");
            String folder = map.getOrDefault("folder", "").toString();
            zipFile(file, zipOutputStream, folder);
        }
        zipOutputStream.close();
        outputStream.close();
        return zipFile;
    }

    /**
     * 压缩文件
     * @param file
     * @param zipOutputStream
     * @throws IOException
     */
    private static void zipFile(File file, ZipOutputStream zipOutputStream, String folder) throws IOException {
        if (file.exists() && file.isFile()) {
            FileInputStream fis = new FileInputStream(file);
            BufferedInputStream bis = new BufferedInputStream(fis);
            String fileName;
            if (!StringUtils.isEmpty(folder)) {
                fileName = folder + "/" + file.getName();
            } else {
                fileName = file.getName();
            }
            ZipEntry entry = new ZipEntry(fileName);

            zipOutputStream.putNextEntry(entry);

            final int MAX_BYTE = 10 * 1024 * 1024;//最大流为10MB
            long streamTotal = 0;//接受流的容量
            int streamNum = 0;//需要分开的流数量
            int leaveByte = 0;//文件剩下的字符数
            byte[] buffer;//byte数据接收文件的数据

            streamTotal = bis.available();//获取流的最大字符数
            streamNum = (int)Math.floor(streamTotal / MAX_BYTE);
            leaveByte = (int)(streamTotal % MAX_BYTE);
            if (streamNum > 0) {
                for (int i=0; i < streamNum; i++) {
                    buffer = new byte[MAX_BYTE];
                    bis.read(buffer, 0, MAX_BYTE);
                    zipOutputStream.write(buffer, 0 , MAX_BYTE);
                }
            }
            //写入剩下的流数据
            buffer = new byte[leaveByte];
            bis.read(buffer, 0, leaveByte);
            zipOutputStream.write(buffer, 0 , leaveByte);
            zipOutputStream.closeEntry();

            bis.close();
            fis.close();
        }
    }
}
